// ------------------------------------------------------------------------- --
// ------------------------------------------------------------------------- --
//                                                                           --
// File................: TESTBNCH.V                                        --
// Function............: Test bench for GENERIC UARTapb                          --
// Version.............: 3.1                                                --
// Last Updated........: 03-08-07                                            --
// Component of........: None                                                --
// Components Required.: UARTapb                                                 --
// Compilation Notes...:                                                     --
//                                                                           --
// Revision History:                                                         --
//   3.1:   Coreconsole version along with APB interface testbench
//   3.00:  added fifo verification
//   2.00:  improve report
//   1.01:  Initial design                                                   --
//   1.00:  Initial design                                                   --
//                                                                           --
//                                                                           --
// ------------------------------------------------------------------------- --
// ------------------------------------------------------------------------- --
`timescale  1ns/1ps

module testbnch ();

`include "../../../../coreparameters.v"

   // The following parameter is used only in the testbench
   parameter FIFO_DEPTH_16 = 0; 

   //-----------------------------------------------
   // component port listings
   //-----------------------------------------------
   //  1=with fifo operation mode
   //  1=with fifo operation mode
   //-----------------------------------------------
   // internal signals
   //-----------------------------------------------
   reg PCLK = 1'b0; 
   reg PRESETN = 1'b0; 
   wire [7:0] UARTstatus1_data_out; 
   wire [7:0] UARTstatus2_data_out; 
   reg UARTconfig1_bit8; 
   reg UARTconfig1_parity_en; 
   reg UARTconfig1_odd_n_even; 
   reg UARTconfig2_bit8; 
   reg UARTconfig2_parity_en; 
   reg UARTconfig2_odd_n_even; 
   reg [7:0] PWDATA1; 
   reg [7:0] PWDATA2; 
   wire txdata1; 
   wire rxdata1; 
   wire txdata2; 
   wire rxdata2; 
   reg PSEL1; 
   reg PSEL2; 
   reg [8:0] count; 
   reg [7:0] tdata[0:511] ; 
   reg [7:0] rdata[0:511] ; 
   reg[7:0] wdata; 
   reg[7:0] wdata1; 
   reg[7:0] wdata2; 
   reg [8:0] rc; 
   reg [8:0] tc; 
   reg [8:0] i; 
   reg [8:0] passflag; 
   reg [7:0] par_err[0:511]; 

   reg PWRITE1;
   reg PWRITE2;
   reg PENABLE1;
   reg PENABLE2;
   reg [8:0] PADDR1;
   reg [8:0] PADDR2;

   wire UARTstatus1_txrdy;
   wire UARTstatus2_rxrdy;
   wire UARTstatus2_parity_err;
   wire UARTstatus2_overflow;

   //-----------------------------------------------
   // 2 Meg baud selection based on 33 MHZ clock
   //-----------------------------------------------

   //-----------------------------------------------
   // drive clock
   //-----------------------------------------------
   always 
   begin
      #15; 
      PCLK = ~PCLK ; 
   end 

   assign rxdata1 = txdata2 ;
   assign rxdata2 = txdata1 ;

   initial 
   begin : xhdl_7
      // ALL TESTS BY DEFAULT TRANSMIT ON UART1 AND RECEIVE ON UART2
      $display("Actel UART Testbench  v3.1 "); 
      $display("--------------------------------------"); 
      $display(" "); 
      //----------------------------------
      // initial control signal conditions
      //----------------------------------
       PSEL1 = 1'b0;
       PSEL2 = 1'b0;
       PWDATA1 = 8'b00000000;
      //-----------------------------------------------
      // drive the resets
      //-----------------------------------------------
      $display("Appling Reset"); 
      PRESETN = 1'b0 ; 
      repeat (10) @(posedge PCLK); 
      @(posedge PCLK); 
      #100; 
      PRESETN = 1'b1 ;
   
     if( (FIXEDMODE == 1'b0) || ((PRG_BIT8 == 1'b1) && (PRG_PARITY == 2'b10) ) )
      begin
      $display("Setting UART1 Operating Mode 8 bit, parity enabled, and even parity"); 
      wdata1 = 8'b00000011; 
      cpu_write1(9'h00C,wdata1);
      $display("Setting UART2 Operating Mode 8 bit, parity enabled, and even parity"); 
      wdata2 = 8'b00000011; 
      cpu_write2(9'h00C,wdata2);
      repeat (10) @(posedge PCLK);
      begin 
        if (FIFO_DEPTH_16 == 1)
	  count = 16;
        else
	  count = 256;
      end
         tc = 0;
         rc = 0;
         $display("Testing Continuous Data Stream UART1 to UART2"); 
         // Initialiase test data
         begin
            for(i = 0; i <= count-1; i = i + 1)
            begin
               tdata[i] = i; 
            end
         end 
         while (rc < count)
          begin
           @(posedge PCLK); 
            if (tc < count)
            begin
               // Firstly See if the Transmitter is ready
               if (UARTstatus1_txrdy == 1'b1)
               begin
                 wdata = tdata[tc]; 
                 cpu_write1(9'h000,wdata);
                 tc = tc + 1; 
               end 
            end 
          // Now See if any received data
            if (UARTstatus2_rxrdy == 1'b1)
            begin
              cpu_read2(004);
              rdata[rc] = UARTstatus2_data_out; 
              rc = rc + 1; 
            end 
          end 
         // Verify that correct data received
          begin 
            passflag = 0; 
            for(i = 0; i <= count - 1; i = i + 1)
            begin
               if (rdata[i] != tdata[i])
                begin
                  $display("THIS TEST IS FAILED\n"); 
	          $finish;
                end 
               else
                begin
                //  $display("CPU write data %04x",tdata[i]);
                //  $display("CPU read data %04x",rdata[i]);
                  passflag = passflag +1;
		  if (passflag == count)
	           begin 
                     $display("THIS TEST IS PASSED\n"); 
	             $display("-----------------------------------------------------"); 
	           end
                end 
            end
          end 
      end 
      
     if( (FIXEDMODE == 1'b0) || ((PRG_BIT8 == 1'b1) && (PRG_PARITY == 2'b01) ) )
      begin
      $display("Setting UART1 Operating Mode 8 bit, parity enabled, and odd parity"); 
      wdata1 = 8'b00000111; 
      cpu_write1(9'h00C,wdata1);
      $display("Setting UART2 Operating Mode 8 bit, parity enabled, and odd parity"); 
      wdata2 = 8'b00000111; 
      cpu_write2(9'h00C,wdata2);
      repeat (10) @(posedge PCLK); 
      begin 
        if (FIFO_DEPTH_16 == 1)
	  count = 16;
        else
	  count = 256;
      end
         tc = 0;
         rc = 0;
         $display("Testing Continuous Data Stream UART1 to UART2"); 
         // Initialiase test data
         begin
            for(i = 0; i <= count-1; i = i + 1)
            begin
               tdata[i] = i; 
            end
         end 
         while (rc < count)
          begin
           @(posedge PCLK); 
            if (tc < count)
            begin
               // Firstly See if the Transmitter is ready
               if (UARTstatus1_txrdy == 1'b1)
               begin
                 wdata = tdata[tc]; 
                 cpu_write1(9'h000,wdata);
                 tc = tc + 1; 
               end 
            end 
          // Now See if any received data
            if (UARTstatus2_rxrdy == 1'b1)
            begin
              cpu_read2(004);
              rdata[rc] = UARTstatus2_data_out; 
              rc = rc + 1; 
            end 
          end 
         // Verify that correct data received
          begin 
            passflag = 0; 
            for(i = 0; i <= count - 1; i = i + 1)
            begin
               if (rdata[i] != tdata[i])
                begin
                  $display("THIS TEST IS FAILED\n"); 
	          $finish;
                end 
               else
                begin
                  passflag = passflag +1;
		  if (passflag == count) 
	           begin 
                     $display("THIS TEST IS PASSED\n"); 
	             $display("-----------------------------------------------------"); 
	           end
                end 
            end
          end 
      end 
         
     if( (FIXEDMODE == 1'b0) || ((PRG_BIT8 == 1'b0) && (PRG_PARITY == 2'b10) ) )
      begin
      $display("Setting UART1 Operating Mode 7 bit, parity enabled, and even parity"); 
      wdata1 = 8'b00000010; 
      cpu_write1(9'h00C,wdata1);
      $display("Setting UART2 Operating Mode 7 bit, parity enabled, and even parity"); 
      wdata2 = 8'b00000010; 
      cpu_write2(9'h00C,wdata2);
      repeat (10) @(posedge PCLK); 
      begin 
        if (FIFO_DEPTH_16 == 1)
	  count = 16;
        else
	  count = 128;
      end
         tc = 0;
         rc = 0;
         $display("Testing Continuous Data Stream UART1 to UART2"); 
         // Initialiase test data
         begin
            for(i = 0; i <= count-1; i = i + 1)
            begin
               tdata[i] = i; 
            end
         end 
         while (rc < count)
          begin
           @(posedge PCLK); 
            if (tc < count)
            begin
               // Firstly See if the Transmitter is ready
               if (UARTstatus1_txrdy == 1'b1)
               begin
                 wdata = tdata[tc]; 
                 cpu_write1(9'h000,wdata);
                 tc = tc + 1; 
               end 
            end 
          // Now See if any received data
            if (UARTstatus2_rxrdy == 1'b1)
            begin
              cpu_read2(004);
              rdata[rc] = UARTstatus2_data_out; 
              rc = rc + 1; 
            end 
          end 
         // Verify that correct data received
          begin 
            passflag = 0; 
            for(i = 0; i <= count - 1; i = i + 1)
            begin
               if (rdata[i] != tdata[i])
                begin
                  $display("THIS TEST IS FAILED\n"); 
	          $finish;
                end 
               else
                begin
                  passflag = passflag +1;
		  if (passflag == count) 
	           begin 
                     $display("THIS TEST IS PASSED\n"); 
	             $display("-----------------------------------------------------"); 
	           end
                end 
            end
          end 
      end 
         
     if( (FIXEDMODE == 1'b0) || ((PRG_BIT8 == 1'b0) && (PRG_PARITY == 2'b01) ) )
      begin
      $display("Setting UART1 Operating Mode 7 bit, parity enabled, and odd parity"); 
      wdata1 = 8'b00000110; 
      cpu_write1(9'h00C,wdata1);
      $display("Setting UART2 Operating Mode 7 bit, parity enabled, and odd parity"); 
      wdata2 = 8'b00000110; 
      cpu_write2(9'h00C,wdata2);
      repeat (10) @(posedge PCLK); 
      begin 
        if (FIFO_DEPTH_16 == 1)
	  count = 16;
        else
	  count = 128;
      end
         tc = 0;
         rc = 0;
         $display("Testing Continuous Data Stream UART1 to UART2"); 
         // Initialiase test data
         begin
            for(i = 0; i <= count-1; i = i + 1)
            begin
               tdata[i] = i; 
            end
         end 
         while (rc < count)
          begin
           @(posedge PCLK); 
            if (tc < count)
            begin
               // Firstly See if the Transmitter is ready
               if (UARTstatus1_txrdy == 1'b1)
               begin
                 wdata = tdata[tc]; 
                 cpu_write1(9'h000,wdata);
                 tc = tc + 1; 
               end 
            end 
          // Now See if any received data
            if (UARTstatus2_rxrdy == 1'b1)
            begin
              cpu_read2(004);
              rdata[rc] = UARTstatus2_data_out; 
              rc = rc + 1; 
            end 
          end 
         // Verify that correct data received
          begin 
            passflag = 0; 
            for(i = 0; i <= count - 1; i = i + 1)
            begin
               if (rdata[i] != tdata[i])
                begin
                  $display("THIS TEST IS FAILED\n"); 
	          $finish;
                end 
               else
                begin
                  passflag = passflag +1;
		  if (passflag == count) 
	           begin 
                     $display("THIS TEST IS PASSED\n"); 
	             $display("-----------------------------------------------------"); 
	           end
                end 
            end
          end 
      end 
         
     if( (FIXEDMODE == 1'b0) || ((PRG_BIT8 == 1'b1) && (PRG_PARITY == 2'b00)  ) )
      begin
      $display("Setting UART1 Operating Mode 8 bit, parity disabled"); 
      wdata1 = 8'b00000101; 
      cpu_write1(9'h00C,wdata1);
      $display("Setting UART2 Operating Mode 8 bit, parity disabled"); 
      wdata2 = 8'b00000101; 
      cpu_write2(9'h00C,wdata2);
      repeat (10) @(posedge PCLK); 
      begin 
        if (FIFO_DEPTH_16 == 1)
	  count = 16;
        else
	  count = 256;
      end
         tc = 0;
         rc = 0;
         $display("Testing Continuous Data Stream UART1 to UART2"); 
         // Initialiase test data
         begin
            for(i = 0; i <= count-1; i = i + 1)
            begin
               tdata[i] = i; 
            end
         end 
         while (rc < count)
          begin
           @(posedge PCLK); 
            if (tc < count)
            begin
               // Firstly See if the Transmitter is ready
               if (UARTstatus1_txrdy == 1'b1)
               begin
                 wdata = tdata[tc]; 
                 cpu_write1(9'h000,wdata);
                 tc = tc + 1; 
               end 
            end 
          // Now See if any received data
            if (UARTstatus2_rxrdy == 1'b1)
            begin
              cpu_read2(004);
              rdata[rc] = UARTstatus2_data_out; 
              rc = rc + 1; 
            end 
          end 
         // Verify that correct data received
          begin 
            passflag = 0; 
            for(i = 0; i <= count - 1; i = i + 1)
            begin
               if (rdata[i] != tdata[i])
                begin
                  $display("THIS TEST IS FAILED\n"); 
	          $finish;
                end 
               else
                begin
                  passflag = passflag +1;
		  if (passflag == count) 
	           begin 
                     $display("THIS TEST IS PASSED\n"); 
	             $display("-----------------------------------------------------"); 
	           end
                end 
            end
          end 
      end 
         
     if( (FIXEDMODE == 1'b0) || ((PRG_BIT8 == 1'b0) && (PRG_PARITY == 2'b00)  ) )
      begin
      $display("Setting UART1 Operating Mode 7 bit, parity disabled"); 
      wdata1 = 8'b00000100; 
      cpu_write1(9'h00C,wdata1);
      $display("Setting UART2 Operating Mode 7 bit, parity disabled"); 
      wdata2 = 8'b00000100; 
      cpu_write2(9'h00C,wdata2);
      repeat (10) @(posedge PCLK); 
      begin 
        if (FIFO_DEPTH_16 == 1)
	  count = 16;
        else
	  count = 128;
      end
         tc = 0;
         rc = 0;
         $display("Testing Continuous Data Stream UART1 to UART2"); 
         // Initialiase test data
         begin
            for(i = 0; i <= count-1; i = i + 1)
            begin
               tdata[i] = i; 
            end
         end 
         while (rc < count)
          begin
           @(posedge PCLK); 
            if (tc < count)
            begin
               // Firstly See if the Transmitter is ready
               if (UARTstatus1_txrdy == 1'b1)
               begin
                 wdata = tdata[tc]; 
                 cpu_write1(9'h000,wdata);
                 tc = tc + 1; 
               end 
            end 
          // Now See if any received data
            if (UARTstatus2_rxrdy == 1'b1)
            begin
              cpu_read2(004);
              rdata[rc] = UARTstatus2_data_out; 
              rc = rc + 1; 
            end 
          end 
         // Verify that correct data received
          begin 
            passflag = 0; 
            for(i = 0; i <= count - 1; i = i + 1)
            begin
               if (rdata[i] != tdata[i])
                begin
                  $display("THIS TEST IS FAILED\n"); 
	          $finish;
                end 
               else
                begin
                  passflag = passflag +1;
		  if (passflag == count) 
	           begin 
                     $display("THIS TEST IS PASSED\n"); 
	             $display("-----------------------------------------------------"); 
	           end
                end 
            end
          end 
      end 
         
      //---------------------------------
      // Testing PARITY ERROR Generations
      //---------------------------------
     if( (FIXEDMODE == 1'b0) || ((PRG_BIT8 == 1'b1) && (PRG_PARITY == 2'b10)  ) )
      begin
      $display("Setting UART1 8bit, parity enabled, and even parity"); 
      // UART1 8bit, parity enabled, and even parity
      wdata1 = 8'b00000011; 
      cpu_write1(9'h00C,wdata1);
      $display("Setting UART2 8bit, parity enabled, and odd parity"); 
      wdata2 = 8'b00000111; 
      cpu_write2(9'h00C,wdata2);
      // UART2 8bit, parity enabled, and odd parity
      repeat (10) @(posedge PCLK); 
      begin 
        if (FIFO_DEPTH_16 == 1)
	  count = 16;
        else
	  count = 256;
      end
         $display("Testing parity error generation"); 
         // Initialiase test data
         tc = 0; 
         rc = 0; 
         i = 0; 
         
         begin
           for(i = 0; i <= count-1; i = i + 1)
             begin
               tdata[i] = i; 
               par_err[i] = 0; 
             end
         end

         while (rc < count)
         begin
            @(posedge PCLK); 
            if (tc < count)
            begin
               // Firstly See if the Transmitter is ready
               if (UARTstatus1_txrdy == 1'b1)
               begin
                  wdata = tdata[tc]; 
                  cpu_write1(9'h000,wdata);
                  tc = tc + 1; 
               end 
            end 
            if (RX_FIFO == 0)
             begin
               // Now See if any received data
               if (UARTstatus2_rxrdy == 1'b1)
                begin
                  par_err[rc] = UARTstatus2_parity_err; 
                  cpu_read2(004);
                  rdata[rc] = UARTstatus2_data_out; 
                  rc = rc + 1; 
                end 
             end
            else
             begin
               if (UARTstatus2_parity_err == 1'b1)
                begin
                  par_err[rc] = UARTstatus2_parity_err; 
                  cpu_read2(004);
                  rdata[rc] = UARTstatus2_data_out; 
                  rc = rc + 1; 
                end 
             end 
         end 
         
         begin 
           passflag = 0; 
           for(i = 0; i <= count - 1; i = i + 1)
             begin
               if (par_err[i] != 1)
                begin 
                  $display("THIS TEST IS FAILED\n"); 
	          $finish;
                end 
               else
                begin
                  passflag = passflag +1;
		  if (passflag == count) 
	           begin 
                     $display("THIS TEST IS PASSED\n"); 
	             $display("-----------------------------------------------------"); 
	           end
                end 
             end
         end 
       end 
    
     if( (FIXEDMODE == 1'b0) || ((PRG_BIT8 == 1'b1) && (PRG_PARITY == 2'b01)  ) )
      begin 
      $display("Setting UART1 8bit, parity enabled, and odd parity"); 
      // UART1 8bit, parity enabled, and odd parity
      wdata1 = 8'b00000111; 
      cpu_write1(9'h00C,wdata1);
      $display("Setting UART2 8bit, parity enabled, and even parity"); 
      wdata2 = 8'b00000011; 
      cpu_write2(9'h00C,wdata2);
      repeat (10) @(posedge PCLK); 
      begin 
        if (FIFO_DEPTH_16 == 1)
	  count = 16;
        else
	  count = 256;
      end
         $display("Testing parity error generation"); 
         // Initialiase test data
         tc = 0; 
         rc = 0; 
         i = 0; 
         
         begin
           for(i = 0; i <= count-1; i = i + 1)
             begin
               tdata[i] = i; 
               par_err[i] = 0; 
             end
         end

         while (rc < count)
         begin
            @(posedge PCLK); 
            if (tc < count)
            begin
               // Firstly See if the Transmitter is ready
               if (UARTstatus1_txrdy == 1'b1)
               begin
                  wdata = tdata[tc]; 
                  cpu_write1(9'h000,wdata);
                  tc = tc + 1; 
               end 
            end 
            if (RX_FIFO == 0)
             begin
               // Now See if any received data
               if (UARTstatus2_rxrdy == 1'b1)
                begin
                  par_err[rc] = UARTstatus2_parity_err; 
                  cpu_read2(004);
                  rdata[rc] = UARTstatus2_data_out; 
                  rc = rc + 1; 
                end 
             end
            else
             begin
               if (UARTstatus2_parity_err == 1'b1)
                begin
                  par_err[rc] = UARTstatus2_parity_err; 
                  cpu_read2(004);
                  rdata[rc] = UARTstatus2_data_out; 
                  rc = rc + 1; 
                end 
             end 
         end 
         
         begin 
           passflag = 0; 
           for(i = 0; i <= count - 1; i = i + 1)
             begin
               if (par_err[i] != 1)
                begin 
                  $display("THIS TEST IS FAILED\n"); 
	          $finish;
                end 
               else
                begin
                  passflag = passflag +1;
		  if (passflag == count) 
	           begin 
                     $display("THIS TEST IS PASSED\n"); 
	             $display("-----------------------------------------------------"); 
	           end
                end 
             end
         end 
       end 
    
     if( (FIXEDMODE == 1'b0) || ((PRG_BIT8 == 1'b0) && (PRG_PARITY == 2'b10)  ) )
      begin 
      $display("Setting UART1 7bit, parity enabled, and even parity"); 
      // UART1 7bit, parity enabled, and even parity
      wdata1 = 8'b00000010; 
      cpu_write1(9'h00C,wdata1);
      $display("Setting UART2 7bit, parity enabled, and odd parity"); 
      // UART2 7bit, parity enabled, and odd parity
      wdata2 = 8'b00000110; 
      cpu_write2(9'h00C,wdata2);
      repeat (10) @(posedge PCLK); 
      begin 
        if (FIFO_DEPTH_16 == 1)
	  count = 16;
        else
	  count = 128;
      end
         $display("Testing parity error generation"); 
         // Initialiase test data
         tc = 0; 
         rc = 0; 
         i = 0; 
         
         begin
           for(i = 0; i <= count-1; i = i + 1)
             begin
               tdata[i] = i; 
               par_err[i] = 0; 
             end
         end

         while (rc < count)
         begin
            @(posedge PCLK); 
            if (tc < count)
            begin
               // Firstly See if the Transmitter is ready
               if (UARTstatus1_txrdy == 1'b1)
               begin
                  wdata = tdata[tc]; 
                  cpu_write1(9'h000,wdata);
                  tc = tc + 1; 
               end 
            end 
            if (RX_FIFO == 0)
             begin
               // Now See if any received data
               if (UARTstatus2_rxrdy == 1'b1)
                begin
                  par_err[rc] = UARTstatus2_parity_err; 
                  cpu_read2(004);
                  rdata[rc] = UARTstatus2_data_out; 
                  rc = rc + 1; 
                end 
             end
            else
             begin
               if (UARTstatus2_parity_err == 1'b1)
                begin
                  par_err[rc] = UARTstatus2_parity_err; 
                  cpu_read2(004);
                  rdata[rc] = UARTstatus2_data_out; 
                  rc = rc + 1; 
                end 
             end 
         end 
         
         begin 
           passflag = 0; 
           for(i = 0; i <= count - 1; i = i + 1)
             begin
               if (par_err[i] != 1)
                begin 
                  $display("THIS TEST IS FAILED\n"); 
	          $finish;
                end 
               else
                begin
                  passflag = passflag +1;
		  if (passflag == count) 
	           begin 
                     $display("THIS TEST IS PASSED\n"); 
	             $display("-----------------------------------------------------"); 
	           end
                end 
             end
         end 
       end 
    
     if( (FIXEDMODE == 1'b0) || ((PRG_BIT8 == 1'b0) && (PRG_PARITY == 2'b01)  ) )
      begin 
      $display("Setting UART1 7bit, parity enabled, and odd parity"); 
      // UART1 7bit, parity enabled, and odd parity
      wdata1 = 8'b00000110; 
      cpu_write1(9'h00C,wdata1);
      $display("Setting UART2 7bit, parity enabled, and even parity"); 
      // UART2 7bit, parity enabled, and even parity
      wdata2 = 8'b00000010; 
      cpu_write2(9'h00C,wdata2);
      repeat (10) @(posedge PCLK); 
      begin 
        if (FIFO_DEPTH_16 == 1)
	  count = 16;
        else
	  count = 256;
      end
         $display("Testing parity error generation"); 
         // Initialiase test data
         tc = 0; 
         rc = 0; 
         i = 0; 
         
         begin
           for(i = 0; i <= count-1; i = i + 1)
             begin
               tdata[i] = i; 
               par_err[i] = 0; 
             end
         end

         while (rc < count)
         begin
            @(posedge PCLK); 
            if (tc < count)
            begin
               // Firstly See if the Transmitter is ready
               if (UARTstatus1_txrdy == 1'b1)
               begin
                  wdata = tdata[tc]; 
                  cpu_write1(9'h000,wdata);
                  tc = tc + 1; 
               end 
            end 
            if (RX_FIFO == 0)
             begin
               // Now See if any received data
               if (UARTstatus2_rxrdy == 1'b1)
                begin
                  par_err[rc] = UARTstatus2_parity_err; 
                  cpu_read2(004);
                  rdata[rc] = UARTstatus2_data_out; 
                  rc = rc + 1; 
                end 
             end
            else
             begin
               if (UARTstatus2_parity_err == 1'b1)
                begin
                  par_err[rc] = UARTstatus2_parity_err; 
                  cpu_read2(004);
                  rdata[rc] = UARTstatus2_data_out; 
                  rc = rc + 1; 
                end 
             end 
         end 
         
         begin 
           passflag = 0; 
           for(i = 0; i <= count - 1; i = i + 1)
             begin
               if (par_err[i] != 1)
                begin 
                  $display("THIS TEST IS FAILED\n"); 
	          $finish;
                end 
               else
                begin
                  passflag = passflag +1;
		  if (passflag == count) 
	           begin 
                     $display("THIS TEST IS PASSED\n"); 
	             $display("-----------------------------------------------------"); 
	           end
                end 
             end
         end 
       end 
    
      //---------------------------------
      // Testing Overflow circuitry
      //---------------------------------
      if( (FIXEDMODE == 1'b0) || ((PRG_BIT8 == 1'b1) && (PRG_PARITY == 2'b01) ) )
       begin
        if (RX_FIFO == 0)
         begin
         $display("Setting UART1 8bit, parity enabled, and odd parity"); 
         wdata1 = 8'b00000111; 
         cpu_write1(9'h00C,wdata1);
         $display("Setting UART2 8bit, parity enabled, and odd parity"); 
         wdata2 = 8'b00000111; 
         cpu_write2(9'h00C,wdata2);
         repeat (100) @(posedge PCLK); 
         $display("Testing for RX data overflow"); 
         cpu_write1(9'h000, 8'b10100101);
         repeat (5) @(posedge PCLK); 
         if (UARTstatus2_overflow == 1'b1)
         begin
            $display("Overflow set too early"); 
         end 
         if (UARTstatus1_txrdy == 1'b1)
         begin
            cpu_write1(9'h000, 8'b01011010);
            @(posedge UARTstatus2_overflow); 
            repeat (10) @(posedge PCLK); 
         end
         else
         begin
            @(posedge UARTstatus1_txrdy); 
            repeat (10) @(posedge PCLK); 
            cpu_write1(9'h000, 8'b01011010);
            @(posedge UARTstatus2_overflow); 
            repeat (10) @(posedge PCLK); 
         end 
         // Get the byte, should be the first one
         if (UARTstatus2_rxrdy == 1'b1)
         begin
            cpu_read2(004);
         end
         else
         begin
            @(posedge UARTstatus2_rxrdy); 
            cpu_read2(004);
         end 
         if (UARTstatus2_data_out != 8'b10100101)
         begin
            $display("THIS TEST IS FAILED\n"); 
	    $finish;
         end
         else
         begin
            $display("THIS TEST IS PASSED\n"); 
	    $display("-----------------------------------------------------"); 
         end 
         repeat (5) @(posedge PCLK); 
         cpu_write1(0, 8'b01011010);
      end 
     end 

     if( (FIXEDMODE == 1'b0) || ((PRG_BIT8 == 1'b0) && (PRG_PARITY == 2'b00)  ) )
     begin
     if (RX_FIFO == 1)
      begin
      $display("Setting UART1 Operating Mode 7 bit, parity disabled"); 
      wdata1 = 8'b00000100; 
      cpu_write1(9'h00C,wdata1);
      $display("Setting UART2 Operating Mode 7 bit, parity disabled"); 
      wdata2 = 8'b00000100; 
      cpu_write2(9'h00C,wdata2);
      repeat (10) @(posedge PCLK);
      begin 
        if (FIFO_DEPTH_16 == 1)
	  count = 16;
        else
	  count = 256;
      end
         tc = 0;
         rc = 0;
         $display("Testing for RX data overflow"); 
         // Initialiase test data
         begin
            for(i = 0; i <= count-1; i = i + 1)
            begin
               tdata[i] = i; 
            end
         end 
         while (tc < count)
          begin
           @(posedge PCLK); 
            if (tc < count)
            begin
               // Firstly See if the Transmitter is ready
               if (UARTstatus1_txrdy == 1'b1)
               begin
                 wdata = tdata[tc]; 
                 cpu_write1(9'h000, wdata);
                 tc = tc + 1; 
               end 
            end 
         end 
          // Now See if any received data
            @(posedge UARTstatus2_overflow); 
            begin
              cpu_read2(004);
              rdata[0] = UARTstatus2_data_out; 
            end 
         // Verify that correct data received
          begin 
               if (rdata[0] != tdata[0])
                begin
                  $display("THIS TEST IS FAILED\n"); 
	          $finish;
                end 
               else
                begin
                  $display("THIS TEST IS PASSED\n"); 
	          $display("-----------------------------------------------------"); 
                end 
          end 
          end 
        end 
      
      $display("END OF SIMULATION"); 
      $display("ALL TESTS ARE PASSED"); 
      $finish; 
   end 


   //----------------------------------------------------------------
   // basic CPU read and write cycles 
   //       
task cpu_write1;
input [8:0] address;
input [7:0] data;
begin
  // $display("CPU Write %04x = %04x",address,data);
  @(negedge PCLK);  
  PWRITE1  = 1'b0;
  @(negedge PCLK);  
  PENABLE1  = 1'b0;
  PSEL1  = 1'b1;
  PADDR1 = address;
  PWDATA1  = data;  
  PWRITE1  = 1'b1;
  @(negedge PCLK);  
  PENABLE1  = 1'b1;
  @(negedge PCLK);  
  PWRITE1  = 1'b0;
  PENABLE1  = 1'b0;
  PSEL1 = 1'b0;
end
endtask

task cpu_write2;
input [8:0] address;
input [7:0] data;
begin
  // $display("CPU Write %04x = %04x",address,data);
  @(negedge PCLK);  
  PWRITE2  = 1'b0;
  @(negedge PCLK);  
  PENABLE2  = 1'b0;
  PSEL2  = 1'b1;
  PADDR2 = address;
  PWDATA2  = data;  
  PWRITE2  = 1'b1;
  @(negedge PCLK);  
  PENABLE2  = 1'b1;
  @(negedge PCLK);  
  PWRITE2  = 1'b0;
  PENABLE2  = 1'b0;
  PSEL2 = 1'b0;
end
endtask

// This task does a cpu read
task cpu_read1;
input [8:0] address;
begin
  @(negedge PCLK);  
  PWRITE1  = 1'b1;
  @(negedge PCLK);  
  PENABLE1  = 1'b0;
  PSEL1  = 1'b1;
  PADDR1 = address;
  PWRITE1  = 1'b0;
  @(negedge PCLK);  
  PENABLE1  = 1'b1;
  @(negedge PCLK);
  PWRITE1  = 1'b1;
  PENABLE1  = 1'b0;
  PSEL1  = 1'b0;
  // $display("CPU Read %04x",address);
end
endtask

task cpu_read2;
input [8:0] address;
begin
  @(negedge PCLK);  
  PWRITE2  = 1'b1;
  @(negedge PCLK);  
  PENABLE2  = 1'b0;
  PSEL2  = 1'b1;
  PADDR2 = address;
  PWRITE2  = 1'b0;
  @(negedge PCLK);  
  PENABLE2  = 1'b1;
  @(negedge PCLK);
  PWRITE2  = 1'b1;
  PENABLE2  = 1'b0;
  PSEL2  = 1'b0;
  // $display("CPU Read %04x = %04x",address, UARTstatus2_data_out);
end
endtask

   //-----------------------------------------------
   // component instantiations
   //-----------------------------------------------

   CoreUARTapb #(.TX_FIFO(TX_FIFO), .RX_FIFO(RX_FIFO), .FAMILY(FAMILY), .BAUD_VALUE(BAUD_VALUE), .FIXEDMODE(FIXEDMODE), .PRG_BIT8(PRG_BIT8), .PRG_PARITY(PRG_PARITY)) make_UART1(.PCLK(PCLK), .PRESETN(PRESETN), .PADDR(PADDR1[4:2]), .PSEL(PSEL1), .PENABLE(PENABLE1), .PWRITE(PWRITE1), .PWDATA(PWDATA1), .PRDATA(UARTstatus1_data_out), .TXRDY(UARTstatus1_txrdy), .RXRDY(UARTstatus1_rxrdy), .PARITY_ERR(UARTstatus1_parity_err), .OVERFLOW(UARTstatus1_overflow), .RX(rxdata1), .TX(txdata1)); 

   CoreUARTapb #(.TX_FIFO(TX_FIFO), .RX_FIFO(RX_FIFO), .FAMILY(FAMILY), .BAUD_VALUE(BAUD_VALUE), .FIXEDMODE(FIXEDMODE), .PRG_BIT8(PRG_BIT8), .PRG_PARITY(PRG_PARITY)) make_UART2(.PCLK(PCLK), .PRESETN(PRESETN), .PADDR(PADDR2[4:2]), .PSEL(PSEL2), .PENABLE(PENABLE2), .PWRITE(PWRITE2), .PWDATA(PWDATA2), .PRDATA(UARTstatus2_data_out), .TXRDY(UARTstatus2_txrdy), .RXRDY(UARTstatus2_rxrdy), .PARITY_ERR(UARTstatus2_parity_err), .OVERFLOW(UARTstatus2_overflow), .RX(rxdata2), .TX(txdata2)); 

endmodule
