//**********************************************************************
// This code was created by SoC Solutions and is free for use with     *
// Actel products.  For support e-mail support@socsolutions.com.       *
//**********************************************************************

`include "socSysDefs.hv"

module socTop (
    pbRstN,
    poRstN,
    flashRstN,
    extClk,
    rs232Atx,
    rs232Arx,
    memAddr,
    memData,
    flashWeN,
    flashHiCeN,
    flashLoCeN,
    flashOeN,
    sramWeN,
    sramCeN,
    sramBsN,
    sramOeN,
    gpio0In,
    gpio0Out,
    gpio1In10,
    gpio1Out10,
    ledOut,
    switchIn,

    XTLCLK,
    PUB,

    AC0,
    AC1,
    AC2,
    AC3,
    AC4,
    AC5,
    AC6,
    AC7,
    AC9,
    AT0,
    AT1,
    AT2,
    AT3,
    AT4,
    AT5,
    AT8,
    AT9,
    ATRETURN0,
    ATRETURN1,
    ATRETURN2,
    ATRETURN4,
    AV0,
    AV1,
    AV2,
    AV3,
    AV4,
    AV5,
    AV6,
    AV7,
    AV8,
    AV9,
    AG0,
    AG1,
    AG2,
    AG3,
    AG4,
    AG5,
    VAREF,

    ICE_TCK,
    ICE_TDI,
    ICE_TMS,
    ICE_TDO,
    ICE_TRTCK,
    ICE_DBGRQ,
    ICE_DBGACK,

    UJTAG_TCK,
    UJTAG_TDI,
    UJTAG_TMS,
    UJTAG_TRSTB,
    UJTAG_TDO,

    pwmOut1,
    pwmOut2,
    testOut
    );

    output [2:0] testOut;

    // resets & clocks
    input  poRstN;                  // external reset
    input  pbRstN;                  // external reset connected to pushbutton
    output flashRstN;               // reset to FLASH
    input  extClk;                  // input clock from socketed osc.

    // UART connections
    output rs232Atx;                // serial transmit line
    input  rs232Arx;                // serial receive line

    output [9:0] ledOut;            // LED outputs
    input  [9:0] switchIn;          // user controllable switches

    // external FLASH/RAM signals
    inout  [31:0] memData;              // bidirectional data to external devices
    output [`N_EXT_ADDR-1:2] memAddr;   // address to external devices
    output flashHiCeN;
    output flashLoCeN;
    output sramCeN;
    output [3:0] sramBsN;           // SRAM Byte Select signals
    output sramWeN;
    output flashWeN;
    output sramOeN;
    output flashOeN;

    // ICE JTAG interface

    input  ICE_TCK;
    input  ICE_TDI;
    input  ICE_TMS;
    output ICE_TDO;
    output ICE_TRTCK;
    input  ICE_DBGRQ;
    output ICE_DBGACK;

    input  UJTAG_TCK;
    input  UJTAG_TDI;
    input  UJTAG_TMS;
    input  UJTAG_TRSTB;
    output UJTAG_TDO;

    output pwmOut1;
    output pwmOut2;

    // Gpio signals

    input  [`GPIO0_WIDTH-1:0] gpio0In;      // Connects the gpio block inputs to the FPGA pins
    output [`GPIO0_WIDTH-1:0] gpio0Out;     // Connects the gpio block outputs to the FPGA pins

    input  gpio1In10;                       // other gpio1 signals go to switches & LEDs
    output gpio1Out10;

    input XTLCLK;
    input PUB;

    input AC0;
    input AC1;
    input AC2;
    input AC3;
    input AC4;
    input AC5;
    input AC6;
    input AC7;
    input AC9;
    input AT0;
    input AT1;
    input AT2;
    input AT3;
    input AT4;
    input AT5;
    input AT8;
    input AT9;
    input ATRETURN0;
    input ATRETURN1;
    input ATRETURN2;
    input ATRETURN4;
    input AV0;
    input AV1;
    input AV2;
    input AV3;
    input AV4;
    input AV5;
    input AV6;
    input AV7;
    input AV8;
    input AV9;
    output AG0;
    output AG1;
    output AG2;
    output AG3;
    output AG4;
    output AG5;
    inout VAREF;

    wire [`GPIO1_WIDTH-1:0] gpio1In;
    wire [`GPIO1_WIDTH-1:0] gpio1Out;

    wire sysClk;        // output of PLL
    wire sysClk_48MHz;
    wire pbRstNi;
    wire poRstNi;

    // Clock, Reset pins use global resources
    pll48to22 pll48to22(
        .POWERDOWN(poRstNi),
        .CLKA(sysClk_48MHz),
        .LOCK(pllLock),
        .GLA(sysClk),
        .OADIVRST(poRstNi)      // ??
    );

    CLKINT ci0 (.A(extClk), .Y(sysClk_48MHz));
    CLKINT ci1 (.A(pbRstN), .Y(pbRstNi));
    CLKINT ci2 (.A(poRstN), .Y(poRstNi));

    // resets
    assign flashRstN = pbRstNi;

    wire extRemap = ~switchIn[9]; // SW=1 = Internal Flash at 0.  SW=0 = remapped (SRAM at 0).

    // ARM debugger

//    wire ICE_nTDOEN;
//    wire ICE_TDOi;
//    assign ICE_TDO = ICE_nTDOEN ? 1'bZ : ICE_TDOi;
//    assign ICE_DBGACK = 1'bZ;

    // keep these inputs from being unused and optimized away
    wire PUCORE;
    wire FPGAGOOD;
//  assign testOut = {PUCORE, FPGAGOOD, ICE_DBGRQ};
    assign testOut = {sysClk, flashRstN, PUCORE | FPGAGOOD};

    // connect LEDs & switches to GPIO & PWMs
    assign ledOut = {pwmOut2 | gpio0Out[9], pwmOut1 | gpio0Out[8], gpio0Out[7:0]};
    assign gpio1In     = {ICE_DBGRQ, ICE_TCK, ICE_TMS, ICE_TDI, gpio1In10, switchIn};
    assign gpio1Out10  = gpio1Out[10];
    assign ICE_TRTCK   = gpio1Out[11];
    assign ICE_TDO     = gpio1Out[12];
    assign ICE_DBGACK  = gpio1Out[13];

    wire flashCeN;
    assign flashHiCeN = flashCeN;
    assign flashLoCeN = flashCeN;

    wire [5:0] pwmUnused;

//  wire VRPU = switchIn[8];
    wire VRPU = 1'b0;   // drive this to shut off voltage regulator

    // Instantiations:

    m1afs_vrpsm m1afs_vrpsm (
        .PUB            (PUB),
        .VRPU           (VRPU),
        .FPGAGOOD       (FPGAGOOD),
        .PUCORE         (PUCORE),
        .RTCPSMMATCH    (RTCPSMMATCH)
    );

    wire [1:0] RTC_MODE;
    m1afs_xtal_rtc m1afs_xtal_rtc (
        .XTL            (XTLCLK),
        .CLKOUT         (RTCCLK),
        .CLKOUTINT      (),
        .SELMODE        (RTCXTLSEL),
        .RTC_MODE       (RTC_MODE)
    );

ccTop ccTop_0(
    // Inputs
    .DDGDON_DDGDON0({1{1'b0}}),
    .DDGDON_DDGDON1({1{1'b0}}),
    .DDGDON_DDGDON2({1{1'b0}}),
    .DDGDON_DDGDON3({1{1'b0}}),
    .DDGDON_DDGDON4({1{1'b0}}),
    .DDGDON_DDGDON5({1{1'b0}}),
    .DDGDON_DDGDON6({1{1'b0}}),
    .DDGDON_DDGDON7({1{1'b0}}),
    .DDGDON_DDGDON8({1{1'b0}}),
    .DDGDON_DDGDON9({1{1'b0}}),
    .NSYSRESET(pbRstNi),
    .RTC_RTCCLK(RTCCLK),
//    .RV_ICE_If_RV_TCK(ICE_TCK),
//    .RV_ICE_If_RV_TDI(ICE_TDI),
//    .RV_ICE_If_RV_TMS(ICE_TMS),
//    .RV_ICE_If_RV_nSRST_IN(pbRstNi),
//    .RV_ICE_If_RV_nTRST(poRstNi),
    .RX(rs232Arx),
    .SYSCLK(sysClk),
    .UJTAG_UJTAG_TCK(UJTAG_TCK),
    .UJTAG_UJTAG_TDI(UJTAG_TDI),
    .UJTAG_UJTAG_TMS(UJTAG_TMS),
    .UJTAG_UJTAG_TRSTB(UJTAG_TRSTB),
    .analog_signals_AC0(AC0),
    .analog_signals_AC1(AC1),
    .analog_signals_AC2(AC2),
    .analog_signals_AC3(AC3),
    .analog_signals_AC4(AC4),
    .analog_signals_AC5(AC5),
    .analog_signals_AC6(AC6),
    .analog_signals_AC7(AC7),
    .analog_signals_AC8({1{1'b0}}),
    .analog_signals_AC9(AC9),
    .analog_signals_AT0(AT0),
    .analog_signals_AT1(AT1),
    .analog_signals_AT2(AT2),
    .analog_signals_AT3(AT3),
    .analog_signals_AT4(AT4),
    .analog_signals_AT5(AT5),
    .analog_signals_AT6({1{1'b0}}),
    .analog_signals_AT7({1{1'b0}}),
    .analog_signals_AT8(AT8),
    .analog_signals_AT9(AT9),
    .analog_signals_ATRETURN0(ATRETURN0),
    .analog_signals_ATRETURN1(ATRETURN1),
    .analog_signals_ATRETURN2(ATRETURN2),
    .analog_signals_ATRETURN3({1{1'b0}}),
    .analog_signals_ATRETURN4(ATRETURN4),
    .analog_signals_AV0(AV0),
    .analog_signals_AV1(AV1),
    .analog_signals_AV2(AV2),
    .analog_signals_AV3(AV3),
    .analog_signals_AV4(AV4),
    .analog_signals_AV5(AV5),
    .analog_signals_AV6(AV6),
    .analog_signals_AV7(AV7),
    .analog_signals_AV8(AV8),
    .analog_signals_AV9(AV9),
    .analog_signals_GNDREF({1{1'b0}}),
    .dataIn(gpio0In),
    .dataIn_01(gpio1In),
    .RemapDefault(extRemap),
    // Outputs
    .DACOUT_DACOUT0(),
    .DACOUT_DACOUT1(),
    .DACOUT_DACOUT2(),
    .DACOUT_DACOUT3(),
    .DACOUT_DACOUT4(),
    .DACOUT_DACOUT5(),
    .DACOUT_DACOUT6(),
    .DACOUT_DACOUT7(),
    .DACOUT_DACOUT8(),
    .DACOUT_DACOUT9(),
    .DATOUT_DATOUT0(),
    .DATOUT_DATOUT1(),
    .DATOUT_DATOUT2(),
    .DATOUT_DATOUT3(),
    .DATOUT_DATOUT4(),
    .DATOUT_DATOUT5(),
    .DATOUT_DATOUT6(),
    .DATOUT_DATOUT7(),
    .DATOUT_DATOUT8(),
    .DATOUT_DATOUT9(),
    .DAVOUT_DAVOUT0(),
    .DAVOUT_DAVOUT1(),
    .DAVOUT_DAVOUT2(),
    .DAVOUT_DAVOUT3(),
    .DAVOUT_DAVOUT4(),
    .DAVOUT_DAVOUT5(),
    .DAVOUT_DAVOUT6(),
    .DAVOUT_DAVOUT7(),
    .DAVOUT_DAVOUT8(),
    .DAVOUT_DAVOUT9(),
    .ExternalMemoryInterface_FlashCSN(flashCeN),
    .ExternalMemoryInterface_FlashOEnN(flashOeN),
    .ExternalMemoryInterface_FlashWEnN(flashWeN),
    .ExternalMemoryInterface_MemAddr(memAddr),      // coreMemCtrl has already removed the 2 LSBs
    .ExternalMemoryInterface_MemReadN(),
    .ExternalMemoryInterface_MemWriteN(),
    .ExternalMemoryInterface_SramByteN(sramBsN),
    .ExternalMemoryInterface_SramCSN(sramCeN),
    .ExternalMemoryInterface_SramClk(),
    .ExternalMemoryInterface_SramOEnN(sramOeN),
    .ExternalMemoryInterface_SramWEnN(sramWeN),
    .FIQ(),
    .IRQ(),
    .PWM({pwmUnused, pwmOut2, pwmOut1}),
    .RTC_RTCMATCH(),
    .RTC_RTCPSMMATCH(RTCPSMMATCH),
    .RTC_RTCXTLMODE0(RTC_MODE[0]),
    .RTC_RTCXTLMODE1(RTC_MODE[1]),
    .RTC_RTCXTLSEL(RTCXTLSEL),
//    .RV_ICE_If_RV_RTCK(ICE_TRTCK),
//    .RV_ICE_If_RV_TDOUT(ICE_TDOi),
//    .RV_ICE_If_RV_nTDOEN(ICE_nTDOEN),
    .TX(rs232Atx),
    .UJTAG_UJTAG_TDO(UJTAG_TDO),
    .analog_signals_AG0(AG0),
    .analog_signals_AG1(AG1),
    .analog_signals_AG2(AG2),
    .analog_signals_AG3(AG3),
    .analog_signals_AG4(AG4),
    .analog_signals_AG5(AG5),
    .analog_signals_AG6(),
    .analog_signals_AG7(),
    .analog_signals_AG8(),
    .analog_signals_AG9(),
    .dataOut(gpio0Out),
    .dataOut_01(gpio1Out),
    // Inouts
    .ExternalMemoryInterface_MemData(memData),
    .analog_signals_VAREF(VAREF)
);

endmodule
