//**********************************************************************
// This code was created by SoC Solutions and is free for use with     *
// Actel products.  For support e-mail support@socsolutions.com.       *
//**********************************************************************

// These definitions are for the CoreAI registers for the Actel Fusion devices.

// registers are 16 bits, hardwired to 32-bit boundaries; byte accesses are not allowed

#ifndef __ANALOG_DEFINES_H
#define __ANALOG_DEFINES_H  1

#define ANALOG_ACM_CTRL_STAT_OFFSET     0x00
#define     ACMWRBUSY_MASK                  0x0010
#define     ACMRDBUSY_MASK                  0x0008
#define     ACMRESETBUSY_MASK               0x0004
#define     ACMRDSTART_MASK                 0x0002
#define     ACMRESET_MASK                   0x0001
#define ANALOG_ACM_ADDR_OFFSET          0x04
#define ANALOG_ACM_DATA_OFFSET          0x08
#define ANALOG_ADC_CTRL1_OFFSET         0x0C
#define     ADC_TVC_MASK                    0xFF00
#define     ADC_RESET_MASK                  0x0040
#define     ADC_PWRDN_MASK                  0x0020
#define     ADC_VAREFSEL_MASK               0x0010
#define     ADC_MODE_MASK                   0x000F
#define         ADC_MODE_10BIT                  0x0000
#define         ADC_MODE_12BIT                  0x0001
#define         ADC_MODE_8BIT                   0x0002
#define         ADC_MODE_NO_INT_PWRDN           0x0004
#define         ADC_MODE_NO_INT_CALIB           0x0008
#define ANALOG_ADC_CTRL2_OFFSET         0x10
#define     ADC_START_MASK                  0x2000
#define     ADC_CHNUMBER_MASK               0x1F00
#define     ADC_STC_MASK                    0x00FF
#define ANALOG_ADC_CTRL3_OFFSET         0x14
#define     ADC_CMSTB_MASK                  0x03FF
#define ANALOG_ADC_CTRL4_OFFSET         0x18
#define     ADC_TMSTBINT_MASK               0x0400
#define     ADC_TMSTB_MASK                  0x03FF
#define ANALOG_ADC_CTRL5_OFFSET         0x1C
#define     ADC_GDON_MASK                   0x03FF
#define ANALOG_ADC_STAT_OFFSET          0x20
#define     ADC_STAT_CALIBRATE_MASK         0x8000
#define     ADC_STAT_SAMPLE_MASK            0x4000
#define     ADC_STAT_BUSY_MASK              0x2000
#define     ADC_STAT_DATAVALID_MASK         0x1000
#define     ADC_STAT_RESULT_MASK            0x0FFF
#define ANALOG_READ_FIFO_DATA_OFFSET    0x24
#define     FIFO_AEMPTY_MASK                0x0008
#define     FIFO_EMPTY_MASK                 0x0004
#define     FIFO_AFULL_MASK                 0x0002
#define     FIFO_FULL_MASK                  0x0001
#define ANALOG_READ_FIFO_STAT_OFFSET    0x28
#define ANALOG_INT_ENB_OFFSET           0x2C
#define ANALOG_INT_STAT_OFFSET          0x30

// ACM (Analog Configuration Mux) interface is configured by writing 
// ANALOG_ACM_ADDR_OFFSET & ANALOG_ACM_DATA_OFFSET.  Before another
// write ANALOG_ACM_CTRL_STAT_LO_OFFSET must be polled until the 
// ACMWRBUSY bit goes away,
// ACM status can be read by writing ANALOG_ACM_ADDR_OFFSET, starting the 
// read by writing ACMRDSTART to ANALOG_ACM_CTRL_STAT_LO_OFFSET, polling
// ANALOG_ACM_CTRL_STAT_LO_OFFSET until the ACMRDBUSY bit goes away,
// and then reading ANALOG_ACM_DATA_OFFSET.

// ACM registers are 8 bits. RTC registers are also accessed through ACM.

#define ACM_AQ0_CTRL    0x01    // for 32-bit accesses
#define ACM_AV0_CTRL    0x01
#define ACM_AC0_CTRL    0x02
#define ACM_AG0_CTRL    0x03
#define ACM_AT0_CTRL    0x04

#define ACM_AQ1_CTRL    0x05    // for 32-bit accesses
#define ACM_AV1_CTRL    0x05
#define ACM_AC1_CTRL    0x06
#define ACM_AG1_CTRL    0x07
#define ACM_AT1_CTRL    0x08

#define ACM_AQ2_CTRL    0x09    // for 32-bit accesses
#define ACM_AV2_CTRL    0x09
#define ACM_AC2_CTRL    0x0A
#define ACM_AG2_CTRL    0x0B
#define ACM_AT2_CTRL    0x0C

#define ACM_AQ3_CTRL    0x0D    // for 32-bit accesses
#define ACM_AV3_CTRL    0x0D
#define ACM_AC3_CTRL    0x0E
#define ACM_AG3_CTRL    0x0F
#define ACM_AT3_CTRL    0x10

#define ACM_AQ4_CTRL    0x11    // for 32-bit accesses
#define ACM_AV4_CTRL    0x11
#define ACM_AC4_CTRL    0x12
#define ACM_AG4_CTRL    0x13
#define ACM_AT4_CTRL    0x14

#define ACM_AQ5_CTRL    0x15    // for 32-bit accesses
#define ACM_AV5_CTRL    0x15
#define ACM_AC5_CTRL    0x16
#define ACM_AG5_CTRL    0x17
#define ACM_AT5_CTRL    0x18

#define ACM_AQ6_CTRL    0x19    // for 32-bit accesses
#define ACM_AV6_CTRL    0x19
#define ACM_AC6_CTRL    0x1A
#define ACM_AG6_CTRL    0x1B
#define ACM_AT6_CTRL    0x1C

#define ACM_AQ7_CTRL    0x1D    // for 32-bit accesses
#define ACM_AV7_CTRL    0x1D
#define ACM_AC7_CTRL    0x1E
#define ACM_AG7_CTRL    0x1F
#define ACM_AT7_CTRL    0x20

#define ACM_AQ8_CTRL    0x21    // for 32-bit accesses
#define ACM_AV8_CTRL    0x21
#define ACM_AC8_CTRL    0x22
#define ACM_AG8_CTRL    0x23
#define ACM_AT8_CTRL    0x24

#define ACM_AQ9_CTRL    0x25    // for 32-bit accesses
#define ACM_AV9_CTRL    0x25
#define ACM_AC9_CTRL    0x26
#define ACM_AG9_CTRL    0x27
#define ACM_AT9_CTRL    0x28

#define RTC_COUNTER0    0x40
#define RTC_COUNTER1    0x41
#define RTC_COUNTER2    0x42
#define RTC_COUNTER3    0x43
#define RTC_COUNTER4    0x44

#define RTC_MATCHREG0   0x48
#define RTC_MATCHREG1   0x49
#define RTC_MATCHREG2   0x4A
#define RTC_MATCHREG3   0x4B
#define RTC_MATCHREG4   0x4C

#define RTC_MATCHBITS0  0x50
#define RTC_MATCHBITS1  0x51
#define RTC_MATCHBITS2  0x52
#define RTC_MATCHBITS3  0x53
#define RTC_MATCHBITS4  0x54

#define RTC_CTRL_STAT   0x58
#define     RTC_RST_MASK            0x80
#define     RTC_CNTR_EN_MASK        0x40
#define     RTC_WR_EN_MAT_MASK      0x20
#define     RTC_XT_MODE_MASK        0x18
#define         RTC_XT_MODE_RC          0x00
#define         RTC_XT_MODE_LG          0x01
#define         RTC_XT_MODE_MG          0x10
#define         RTC_XT_MODE_HG          0x11
#define     RTC_RST_CNT_OMAT_MASK   0x04
#define     RTC_NRST_CNT_MASK       0x02
#define     RTC_XTAL_EN_MASK        0x01
#define RTC_TEST_REG    0x59

//#define MAX_ACM_POLLS   4
#define MAX_ACM_POLLS   400
//#define MAX_ADC_POLLS   16
#define MAX_ADC_POLLS   5000

// analog mux channels:

#define AMUX_CH_VCCA        0
#define AMUX_CH_AV0         1
#define AMUX_CH_AC0         2
#define AMUX_CH_AT0         3
#define AMUX_CH_AV1         4
#define AMUX_CH_AC1         5
#define AMUX_CH_AT1         6
#define AMUX_CH_AV2         7
#define AMUX_CH_AC2         8
#define AMUX_CH_AT2         9
#define AMUX_CH_AV3         10
#define AMUX_CH_AC3         11
#define AMUX_CH_AT3         12
#define AMUX_CH_AV4         13
#define AMUX_CH_AC4         14
#define AMUX_CH_AT4         15
#define AMUX_CH_AV5         16
#define AMUX_CH_AC5         17
#define AMUX_CH_AT5         18
#define AMUX_CH_AV6         19
#define AMUX_CH_AC6         20
#define AMUX_CH_AT6         21
#define AMUX_CH_AV7         22
#define AMUX_CH_AC7         23
#define AMUX_CH_AT7         24
#define AMUX_CH_AV8         25
#define AMUX_CH_AC8         26
#define AMUX_CH_AT8         27
#define AMUX_CH_AV9         28
#define AMUX_CH_AC9         29
#define AMUX_CH_AT9         30
#define AMUX_CH_INT_TEMP    31

#define MAX_CH              32
#define AMUX_CH_NONE        32  // used as array terminator

#define ANALOG_CH_TYPE_VOLTAGE  0
#define ANALOG_CH_TYPE_CURRENT  1
#define ANALOG_CH_TYPE_TEMP     2

#define ADC_MODE  ADC_MODE_10BIT    // change this for more or less bits

#define ANALOG_AG0          0x0001  // GDON register bits to enable gate drivers
#define ANALOG_AG1          0x0002
#define ANALOG_AG2          0x0004
#define ANALOG_AG3          0x0008
#define ANALOG_AG4          0x0010
#define ANALOG_AG5          0x0020
#define ANALOG_AG6          0x0040
#define ANALOG_AG7          0x0080
#define ANALOG_AG8          0x0100
#define ANALOG_AG9          0x0200

#define ANALOG_DIG_HI_THRESH    0x0200
#define ANALOG_DIG_LO_THRESH    0x0080

struct analog_ch_desc {
    uint8 amux_ch;
    uint8 avg_samples;
    uint16 ch_type;
    uint16 result;
};

struct internal_ch_desc {
    uint8 amux_ch;
    uint8 avg_samples;  // 0 = just add data to buffer & go on to next measurement
    uint16 ch_type;
    uint16 *meas_buf_start;
    uint16 *meas_buf_ptr;
    uint16 *result_ptr;
};

#define MAX_AVG     128  // must be a power of 2 & <= 128

#endif

